@@ -12,6 +12,7 @@ from pulsar_qcm.pulsar_qcm_ifc import pulsar_qcm_ifc
 from qcodes                    import validators as vals
 from qcodes                    import Instrument
 from jsonschema                import validate
+from functools                 import partial
 import json
 
 #-- class ---------------------------------------------------------------------
@@ -67,7 +68,7 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
 
         #Set JSON schema to validate JSON file with
         self._wave_and_prog_json_schema = {"title":       "Sequencer waveforms and program container",
-                                           "description": "Contains both all waveforms and program required for a sequence.",
+                                           "description": "Contains both all waveforms and a program required for a sequence.",
                                            "type":        "object",
                                            "required":    ["program", "waveforms"],
                                            "properties": {
@@ -77,17 +78,11 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                                                },
                                                "waveforms": {
                                                    "description": "Waveform dictionary containing one or multiple AWG waveform(s).",
-                                                   "type":        "object",
-                                                   "properties": {
-                                                       "awg": {
-                                                            "description": "Waveform dictionary containing one or multiple AWG waveform(s).",
-                                                            "type":        "object"
-                                                       },
-                                                   }
+                                                   "type":        "object"
                                                }
                                            }}
         self._wave_json_schema = {"title":       "Waveform container",
-                                  "description": "Waveform dictionary a single waveform.",
+                                  "description": "Waveform dictionary for a single waveform.",
                                   "type":        "object",
                                   "required":    ["data"],
                                   "properties": {
@@ -150,8 +145,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "sync_en"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "sync_en")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "sync_en"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "sync_en")
             )
 
             self.add_parameter(
@@ -162,8 +157,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(-300e6, 300e6),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "freq_hz"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "freq_hz")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "freq_hz"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "freq_hz")
             )
 
             self.add_parameter(
@@ -174,8 +169,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(0, 360),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "phase_offs_degree"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "phase_offs_degree")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "phase_offs_degree"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "phase_offs_degree")
             )
 
             self.add_parameter(
@@ -186,8 +181,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "mrk_ovr_en"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "mrk_ovr_en")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "mrk_ovr_en"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "mrk_ovr_en")
             )
 
             self.add_parameter(
@@ -198,8 +193,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(0, 15),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "mrk_ovr_val"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "mrk_ovr_val")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "mrk_ovr_val"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "mrk_ovr_val")
             )
 
             self.add_parameter(
@@ -209,7 +204,7 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Strings(),
                 set_parser = str,
                 get_parser = str,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_waveforms_and_program, seq_idx),
+                set_cmd    = partial(self._set_sequencer_waveforms_and_program, seq_idx),
             )
 
 
@@ -223,8 +218,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0")
             )
 
             self.add_parameter(
@@ -235,8 +230,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1")
             )
 
             self.add_parameter(
@@ -247,8 +242,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**10-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0")
             )
 
             self.add_parameter(
@@ -259,8 +254,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**10-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1")
             )
 
             self.add_parameter(
@@ -271,8 +266,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**16-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0")
             )
 
             self.add_parameter(
@@ -283,8 +278,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**16-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1")
             )
 
             self.add_parameter(
@@ -295,8 +290,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "gain_awg_path_0_float"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "gain_awg_path_0_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "gain_awg_path_0_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "gain_awg_path_0_float")
             )
 
             self.add_parameter(
@@ -307,8 +302,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "gain_awg_path_1_float"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "gain_awg_path_1_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "gain_awg_path_1_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "gain_awg_path_1_float")
             )
 
             self.add_parameter(
@@ -319,8 +314,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "offset_awg_path_0_float"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "offset_awg_path_0_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "offset_awg_path_0_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "offset_awg_path_0_float")
             )
 
             self.add_parameter(
@@ -331,8 +326,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "offset_awg_path_1_float"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "offset_awg_path_1_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "offset_awg_path_1_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "offset_awg_path_1_float")
             )
 
             self.add_parameter(
@@ -343,8 +338,8 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qcm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "mod_en_awg"),
-                get_cmd    = pulsar_qcm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "mod_en_awg")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "mod_en_awg"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "mod_en_awg")
             )
 
     #--------------------------------------------------------------------------
@@ -442,65 +437,14 @@ class pulsar_qcm_qcodes(pulsar_qcm_ifc, Instrument):
             with open(file_name, 'r') as file:
                 wave_and_prog_dict = json.load(file)
                 validate(wave_and_prog_dict, self._wave_and_prog_json_schema)
-                if "awg" in wave_and_prog_dict["waveforms"]:
-                    for name in wave_and_prog_dict["waveforms"]["awg"]:
-                        validate(wave_and_prog_dict["waveforms"]["awg"][name], self._wave_json_schema)
+                for name in wave_and_prog_dict["waveforms"]:
+                    validate(wave_and_prog_dict["waveforms"][name], self._wave_json_schema)
                 self._delete_waveforms(sequencer)
                 self._add_waveforms(sequencer, wave_and_prog_dict["waveforms"])
                 self._set_sequencer_program(sequencer, wave_and_prog_dict["program"])
         except:
             raise
 
-    #--------------------------------------------------------------------------
-    @staticmethod
-    def _gen_set_func_par(func, *args):
-        """
-        Generate set function with fixed parameters.
-
-        Parameters
-        ----------
-        func : callable
-            Function to generate set function for.
-        args
-            Function arguments.
-
-        Returns
-        ----------
-
-        Raises
-        ----------
-        """
-
-        def set_func(val):
-            return func(*args, val)
-        return set_func
-
-    #--------------------------------------------------------------------------
-    @staticmethod
-    def _gen_get_func_par(func, *args):
-        """
-        Generate get function with fixed parameters.
-
-        Parameters
-        ----------
-        func : callable
-            Function to generate set function for.
-        args
-            Function arguments.
-
-        Returns
-        ----------
-        val
-            Function return value.
-
-        Raises
-        ----------
-        """
-
-        def get_func():
-            return func(*args)
-        return get_func
-
 #-- class ---------------------------------------------------------------------
 
 class pulsar_qcm(pulsar_qcm_qcodes):
@@ -568,7 +512,7 @@ class pulsar_qcm_dummy(pulsar_qcm_qcodes):
         """
 
         #Create transport layer (socket interface)
-        transport_inst = pulsar_dummy_transport(pulsar_qcm_ifc._get_sequencer_cfg_format())
+        transport_inst = pulsar_dummy_transport('', pulsar_qcm_ifc._get_sequencer_cfg_format())
 
         #Initialize parent classes.
         super(pulsar_qcm_dummy, self).__init__(name, transport_inst, debug)
