@@ -148,7 +148,7 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
 
         Returns
         ----------
-        str
+        dict
             Dictionary containing general status and corresponding flags:
 
             :Status:
@@ -485,7 +485,7 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
 
         Returns
         ----------
-        str
+        list
             Concatinated list of strings separated by the semicolon character. Status is indicated by one status string and an optional number of flags respectively ordered as:
 
             :Status:
@@ -500,8 +500,8 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
                 - FORCED STOP: Sequencer was stopped while still running.
                 - SEQUENCE PROCESSOR Q1 ILLEGAL INSTRUCTION: Classical sequencer part executed an unknown instruction.
                 - SEQUENCE PROCESSOR RT EXEC ILLEGAL INSTRUCTION: Real-time sequencer part executed an unknown instruction.
-                - AWG WAVE PLAYBACK INVALID PATH 0: AWG path 0 tried to play an unknown waveform.
-                - AWG WAVE PLAYBACK INVALID PATH 1: AWG path 1 tried to play an unknown waveform.
+                - AWG WAVE PLAYBACK INDEX INVALID PATH 0: AWG path 0 tried to play an unknown waveform.
+                - AWG WAVE PLAYBACK INDEX INVALID PATH 1: AWG path 1 tried to play an unknown waveform.
                 - CLOCK INSTABILITY: Clock source instability occurred.
 
         Raises
@@ -584,28 +584,20 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
         except:
             raise
 
-    #--------------------------------------------------------------------------
-    def _get_awg_waveform_data(self, sequencer, name, start=0, size=2**31):
+    #---------------------------------------------------------------------------
+    def _get_awg_waveforms(self, sequencer):
         """
-        Get waveform data of waveform in AWG waveform list of indexed sequencer. If an invalid sequencer index is given or if a non-existing waveform
-        name is given, an error is set in system error. The start and size arguments can be used to return a section of the waveform. If the start argument
-        is not given, zero is used as the default start sample. If the size argument is not given, all samples starting from the start sample are returned.
+        Get all waveforms in waveform list of indexed sequencer's AWG path. If an invalid sequencer index is given, an error is set in system error.
 
         Parameters
         ----------
         sequencer : int
             Sequencer index.
-        name : str
-            Waveform name.
-        start : int
-            First sample within the waveform to be returned.
-        size : int
-            Number of samples starting from the start sample to be returned.
 
         Returns
         ----------
-        list
-            List of floats in the range of 1.0 to -1.0 representing the waveform.
+        dict
+            Dictionary with waveforms.
 
         Raises
         ----------
@@ -616,48 +608,44 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
             All errors are read from system error and listed in the exception.
         """
 
+        #Check input types.
+        self._check_in_type(locals(), ['int'])
+
+        #SCPI call
         try:
-            return super()._get_awg_waveform_data(sequencer, name, start, size)
-        except:
-            raise
+            num_waveforms = struct.unpack('I', self._read_bin('SEQuencer{}:AWG:WLISt?'.format(sequencer), False))[0]
+            if num_waveforms == 0:
+                self._flush_line_end()
 
-    #--------------------------------------------------------------------------
-    def _get_awg_waveform_names(self, sequencer):
-        """
-        Get all waveform names in AWG waveform list of indexed sequencer's AWG path. If an invalid sequencer index is given, an error is set in system error.
+            waveform_dict = {}
+            for wave_it in range(0, num_waveforms):
+                #Get name and index
+                name  = str(self._read_bin('', False), 'utf-8')
+                index = struct.unpack('I', self._read_bin('', False))[0]
 
-        Parameters
-        ----------
-        sequencer : int
-            Sequencer index.
+                #Get data
+                if wave_it < (num_waveforms - 1):
+                    data = self._read_bin('', False)
+                else:
+                    data = self._read_bin('', True)
+                data = struct.unpack('f'*int(len(data)/4), data)
 
-        Returns
-        ----------
-        int
-            Number of waveforms.
+                #Add to dictionary
+                waveform_dict[name] = {"index": index,
+                                       "data":  list(data)}
 
-        Raises
-        ----------
-        Exception
-            Invalid input parameter type.
-        Exception
-            An error is reported in system error and debug <= 1.
-            All errors are read from system error and listed in the exception.
-        """
-
-        try:
-            names     = super()._get_awg_waveform_names(sequencer)
-            name_list = names.split(';')[:-1] if names != '' else []
-            return name_list
-        except:
-            raise
+            return waveform_dict
+        except Exception as err:
+            self._check_error_queue(err)
+        finally:
+            self._check_error_queue()
 
 
 
     #--------------------------------------------------------------------------
     def _add_waveforms(self, sequencer, waveform_dict):
         """
-        Add all waveforms in JSON compatible dictionary to AWG waveform list of indexed sequencer.
+        Add all waveforms in JSON compatible dictionary to the AWG waveform list of indexed sequencer.
         The dictionary must be structured as follows:
 
         - awg
@@ -684,27 +672,26 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
         Exception
             An error is reported in system error and debug <= 1.
             All errors are read from system error and listed in the exception.
-        Exception
+        KeyError
             Missing waveform data of waveform in dictionary.
         """
 
         try:
-            if "awg" in waveform_dict:
-                for name in waveform_dict["awg"]:
-                    if "data" in waveform_dict["awg"][name]:
-                        if "index" in waveform_dict["awg"][name]:
-                            self._add_awg_waveform(sequencer, name, waveform_dict["awg"][name]["data"], waveform_dict["awg"][name]["index"])
-                        else:
-                            self._add_awg_waveform(sequencer, name, waveform_dict["awg"][name]["data"])
+            for name in waveform_dict:
+                if "data" in waveform_dict[name]:
+                    if "index" in waveform_dict[name]:
+                        self._add_awg_waveform(sequencer, name, waveform_dict[name]["data"], waveform_dict[name]["index"])
                     else:
-                        raise Exception("Missing data key for {} in AWG waveform dictionary".format(name))
+                        self._add_awg_waveform(sequencer, name, waveform_dict[name]["data"])
+                else:
+                    raise KeyError("Missing data key for {} in AWG waveform dictionary".format(name))
         except:
             raise
 
     #--------------------------------------------------------------------------
     def _delete_waveforms(self, sequencer):
         """
-        Delete all waveforms in AWG waveform list of indexed sequencer.
+        Delete all waveforms in the AWG waveform list of indexed sequencer.
 
         Parameters
         ----------
@@ -731,15 +718,13 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
     #--------------------------------------------------------------------------
     def get_waveforms(self, sequencer):
         """
-        Get all waveforms in AWG waveform list of indexed sequencer.
+        Get all waveforms in the AWG waveform list of indexed sequencer.
         The returned dictionary is structured as follows:
 
-        - awg
+        - name: waveform name.
 
-            - name: waveform name.
-
-                - data: waveform samples in a range of 1.0 to -1.0.
-                - index: optional waveform index used by the sequencer Q1ASM program to refer to the waveform.
+            - data: waveform samples in a range of 1.0 to -1.0.
+            - index: waveform index used by the sequencer Q1ASM program to refer to the waveform.
 
         Parameters
         ----------
@@ -761,14 +746,6 @@ class pulsar_qcm_ifc(pulsar_qcm_scpi_ifc):
         """
 
         try:
-            waveform_dict = {"awg": {}}
-
-            name_list = self._get_awg_waveform_names(sequencer)
-            for name in name_list:
-                waveform_dict["awg"][name] = {}
-                waveform_dict["awg"][name]["data"]  = self._get_awg_waveform_data(sequencer, name, 0, 2**31-1)
-                waveform_dict["awg"][name]["index"] = self._get_awg_waveform_index(sequencer, name)
-
-            return waveform_dict
+            return self._get_awg_waveforms(sequencer)
         except:
             raise
