@@ -12,6 +12,7 @@ from pulsar_qrm.pulsar_qrm_ifc import pulsar_qrm_ifc
 from qcodes                    import validators as vals
 from qcodes                    import Instrument
 from jsonschema                import validate
+from functools                 import partial
 import json
 
 #-- class ---------------------------------------------------------------------
@@ -67,31 +68,29 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
 
         #Set JSON schema to validate JSON file with
         self._wave_and_prog_json_schema = {"title":       "Sequencer waveforms and program container",
-                                           "description": "Contains both all waveforms and program required for a sequence.",
+                                           "description": "Contains both all waveforms, weights and acquisitions and a program required for a sequence.",
                                            "type":        "object",
-                                           "required":    ["program", "waveforms"],
+                                           "required":    ["program", "waveforms", "weights", "acquisitions"],
                                            "properties": {
                                                "program": {
                                                    "description": "Sequencer assembly program in string format.",
                                                    "type":        "string"
                                                },
                                                "waveforms": {
-                                                   "description": "Waveform dictionary containing one or multiple AWG and/or acquisition waveform(s).",
+                                                   "description": "Waveform dictionary containing one or multiple AWG waveform(s).",
                                                    "type":        "object",
-                                                   "properties": {
-                                                       "awg": {
-                                                            "description": "Waveform dictionary containing one or multiple AWG waveform(s).",
-                                                            "type":        "object"
-                                                       },
-                                                       "acq": {
-                                                            "description": "Waveform dictionary containing one or multiple acquisition waveform(s).",
-                                                            "type":        "object"
-                                                       }
-                                                   }
+                                               },
+                                               "weights": {
+                                                   "description": "Weight dictionary containing one or multiple acquisition weights(s).",
+                                                   "type":        "object",
+                                               },
+                                               "acquisitions": {
+                                                   "description": "Acquisition dictionary containing information about one or multiple acquisition(s).",
+                                                   "type":        "object"
                                                }
                                            }}
-        self._wave_json_schema = {"title":       "Waveform container",
-                                  "description": "Waveform dictionary a single waveform.",
+        self._wave_json_schema = {"title":       "Waveform/weight container",
+                                  "description": "Waveform/weight dictionary for a single waveform.",
                                   "type":        "object",
                                   "required":    ["data"],
                                   "properties": {
@@ -105,6 +104,21 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                                       }
                                   }}
 
+        self._acq_json_schema = {"title":       "Acquisition container",
+                                 "description": "Acquisition dictionary for a single acquisition.",
+                                 "type":        "object",
+                                 "required":    ["num_bins"],
+                                 "properties": {
+                                     "num_bins": {
+                                         "description": "Number of bins in acquisition.",
+                                         "type":        "number"
+                                     },
+                                     "index": {
+                                         "description": "Optional waveform index number.",
+                                         "type":        "number"
+                                     }
+                                 }}
+
         #Add QCoDeS parameters
         self.add_parameter(
             "reference_source",
@@ -168,6 +182,92 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
             get_cmd    = self._get_in_amp_gain_1
         )
 
+        #--Scope acquisition settings------------------------------------------
+        self.add_parameter(
+            "scope_acq_trigger_mode_path0",
+            label       = "Scope acquisition trigger mode for input path 0.",
+            docstring   = "Sets/gets scope acquisition trigger mode for input path 0 ('sequencer' = triggered by sequencer, 'level' = triggered by input level).",
+            unit        = '',
+            vals        = vals.Bool(),
+            val_mapping = {"level": True, "sequencer": False},
+            set_parser  = bool,
+            get_parser  = bool,
+            set_cmd     = partial(self._set_acq_scope_config_val, "trig_mode_acq_path_0"),
+            get_cmd     = partial(self._get_acq_scope_config_val, "trig_mode_acq_path_0")
+        )
+        
+        self.add_parameter(
+            "scope_acq_trigger_mode_path1",
+            label       = "Scope acquisition trigger mode for input path 0.",
+            docstring   = "Sets/gets scope acquisition trigger mode for input path 1 ('sequencer' = triggered by sequencer, 'level' = triggered by input level).",
+            unit        = '',
+            vals        = vals.Bool(),
+            val_mapping = {"level": True, "sequencer": False},
+            set_parser  = bool,
+            get_parser  = bool,
+            set_cmd     = partial(self._set_acq_scope_config_val, "trig_mode_acq_path_1"),
+            get_cmd     = partial(self._get_acq_scope_config_val, "trig_mode_acq_path_1")
+        )
+
+        self.add_parameter(
+            "scope_acq_trigger_level_path0",
+            label      = "Scope acquisition trigger level when using input level trigger mode for input path 0.",
+            docstring  = "Sets/gets scope acquisition trigger level when using input level trigger mode for input path 0.",
+            unit       = '',
+            vals       = vals.Numbers(-1.0, 1.0),
+            set_parser = float,
+            get_parser = float,
+            set_cmd    = partial(self._set_acq_scope_config_val, "trig_lvl_acq_path_0_float"),
+            get_cmd    = partial(self._get_acq_scope_config_val, "trig_lvl_acq_path_0_float")
+        )
+
+        self.add_parameter(
+            "scope_acq_trigger_level_path1",
+            label      = "Scope acquisition trigger level when using input level trigger mode for input path 1.",
+            docstring  = "Sets/gets scope acquisition trigger level when using input level trigger mode for input path 1.",
+            unit       = '',
+            vals       = vals.Numbers(-1.0, 1.0),
+            set_parser = float,
+            get_parser = float,
+            set_cmd    = partial(self._set_acq_scope_config_val, "trig_lvl_acq_path_1_float"),
+            get_cmd    = partial(self._get_acq_scope_config_val, "trig_lvl_acq_path_1_float")
+        )
+        
+        self.add_parameter(
+            "scope_acq_sequencer_select",
+            label       = "Scope acquisition sequencer select for when using sequencer trigger mode.",
+            docstring   = "Sets/gets sequencer select that specifies which sequencer triggers the scope acquisition when using sequencer trigger mode.",
+            unit        = '',
+            vals        = vals.Numbers(0, self._num_sequencers),
+            set_parser  = int,
+            get_parser  = int,
+            set_cmd     = partial(self._set_acq_scope_config_val, "trig_mode_acq_path_0"),
+            get_cmd     = partial(self._get_acq_scope_config_val, "trig_mode_acq_path_0")
+        )
+
+        self.add_parameter(
+            "scope_acq_avg_mode_en_path0",
+            label      = "Scope acquisition averaging mode enable for input path 0.",
+            docstring  = "Sets/gets scope acquisition averaging mode enable for input path 0.",
+            unit       = '',
+            vals       = vals.Bool(),
+            set_parser = bool,
+            get_parser = bool,
+            set_cmd    = partial(self._set_acq_scope_config_val, "avg_en_acq_path_0"),
+            get_cmd    = partial(self._get_acq_scope_config_val, "avg_en_acq_path_0")
+        )
+        self.add_parameter(
+            "scope_acq_avg_mode_en_path1",
+            label      = "Scope acquisition averaging mode enable for input path 0.",
+            docstring  = "Sets/gets scope acquisition averaging mode enable for input path 0.",
+            unit       = '',
+            vals       = vals.Bool(),
+            set_parser = bool,
+            get_parser = bool,
+            set_cmd    = partial(self._set_acq_scope_config_val, "avg_en_acq_path_1"),
+            get_cmd    = partial(self._get_acq_scope_config_val, "avg_en_acq_path_1")
+        )
+
         for seq_idx in range(0, self._num_sequencers):
             #--Sequencer settings----------------------------------------------
             self.add_parameter(
@@ -178,8 +278,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "sync_en"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "sync_en")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "sync_en"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "sync_en")
             )
 
             self.add_parameter(
@@ -190,8 +290,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(-300e6, 300e6),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "freq_hz"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "freq_hz")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "freq_hz"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "freq_hz")
             )
 
             self.add_parameter(
@@ -202,8 +302,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(0, 360),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "phase_offs_degree"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "phase_offs_degree")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "phase_offs_degree"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "phase_offs_degree")
             )
 
             self.add_parameter(
@@ -214,8 +314,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "mrk_ovr_en"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "mrk_ovr_en")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "mrk_ovr_en"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "mrk_ovr_en")
             )
 
             self.add_parameter(
@@ -226,18 +326,18 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(0, 15),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "mrk_ovr_val"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "mrk_ovr_val")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "mrk_ovr_val"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "mrk_ovr_val")
             )
 
             self.add_parameter(
                 "sequencer{}_waveforms_and_program".format(seq_idx),
-                label      = "Sequencer {} AWG and acquistion waveforms and ASM program.".format(seq_idx),
-                docstring  = "Sets sequencer {} AWG and acquistion waveforms and ASM program. Valid input is a string representing the JSON filename.".format(seq_idx),
+                label      = "Sequencer {} AWG waveforms, acquistion weights, acquisitions and ASM program.".format(seq_idx),
+                docstring  = "Sets sequencer {} AWG waveforms, acquistion weights, acquisitions and ASM program. Valid input is a string representing the JSON filename.".format(seq_idx),
                 vals       = vals.Strings(),
                 set_parser = str,
                 get_parser = str,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_waveforms_and_program, seq_idx),
+                set_cmd    = partial(self._set_sequencer_waveforms_and_program, seq_idx),
             )
 
 
@@ -251,8 +351,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_0")
             )
 
             self.add_parameter(
@@ -263,8 +363,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_en_awg_path_1")
             )
 
             self.add_parameter(
@@ -275,8 +375,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**10-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_0")
             )
 
             self.add_parameter(
@@ -287,8 +387,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**10-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "cont_mode_waveform_idx_awg_path_1")
             )
 
             self.add_parameter(
@@ -299,8 +399,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**16-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_0")
             )
 
             self.add_parameter(
@@ -311,8 +411,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(0, 2**16-1),
                 set_parser = int,
                 get_parser = int,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "upsample_rate_awg_path_1")
             )
 
             self.add_parameter(
@@ -323,8 +423,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "gain_awg_path_0_float"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "gain_awg_path_0_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "gain_awg_path_0_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "gain_awg_path_0_float")
             )
 
             self.add_parameter(
@@ -335,8 +435,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "gain_awg_path_1_float"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "gain_awg_path_1_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "gain_awg_path_1_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "gain_awg_path_1_float")
             )
 
             self.add_parameter(
@@ -347,8 +447,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "offset_awg_path_0_float"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "offset_awg_path_0_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "offset_awg_path_0_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "offset_awg_path_0_float")
             )
 
             self.add_parameter(
@@ -359,8 +459,8 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Numbers(-1.0, 1.0),
                 set_parser = float,
                 get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "offset_awg_path_1_float"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "offset_awg_path_1_float")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "offset_awg_path_1_float"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "offset_awg_path_1_float")
             )
 
             self.add_parameter(
@@ -371,87 +471,123 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "mod_en_awg"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "mod_en_awg")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "mod_en_awg"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "mod_en_awg")
             )
 
 
 
             #--Acquisition settings--------------------------------------------
             self.add_parameter(
-                "sequencer{}_trigger_mode_acq_path0".format(seq_idx),
-                label       = "Sequencer {} trigger mode for acquisition path 0.".format(seq_idx),
-                docstring   = "Sets/gets sequencer {} trigger mode for acquisition path 0 ('sequencer' = triggered by sequencer, 'level' = triggered by input level).".format(seq_idx),
-                unit        = '',
-                vals        = vals.Bool(),
-                val_mapping = {"level": True, "sequencer": False},
-                set_parser  = bool,
-                get_parser  = bool,
-                set_cmd     = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "trig_mode_acq_path_0"),
-                get_cmd     = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "trig_mode_acq_path_0")
-            )
-
-            self.add_parameter(
-                "sequencer{}_trigger_mode_acq_path1".format(seq_idx),
-                label       = "Sequencer {} trigger mode for acquisition path 1.".format(seq_idx),
-                docstring   = "Sets/gets sequencer {} trigger mode for acquisition path 1 ('sequencer' = triggered by sequencer, 'level' = triggered by input level).".format(seq_idx),
-                unit        = '',
-                vals        = vals.Bool(),
-                val_mapping = {"level": True, "sequencer": False},
-                set_parser  = bool,
-                get_parser  = bool,
-                set_cmd     = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "trig_mode_acq_path_1"),
-                get_cmd     = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "trig_mode_acq_path_1")
-            )
-
-            self.add_parameter(
-                "sequencer{}_trigger_level_acq_path0".format(seq_idx),
-                label      = "Sequencer {} trigger level when using input level trigger mode for acquisition path 0.".format(seq_idx),
-                docstring  = "Sets/gets sequencer {} trigger level when using input level trigger mode for acquisition path 0.".format(seq_idx),
-                unit       = '',
-                vals       = vals.Numbers(-1.0, 1.0),
-                set_parser = float,
-                get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "trig_lvl_acq_path_0_float"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "trig_lvl_acq_path_0_float")
-            )
-
-            self.add_parameter(
-                "sequencer{}_trigger_level_acq_path1".format(seq_idx),
-                label      = "Sequencer {} trigger level when using input level trigger mode for acquisition path 1.".format(seq_idx),
-                docstring  = "Sets/gets sequencer {} trigger level when using input level trigger mode for acquisition path 1.".format(seq_idx),
-                unit       = '',
-                vals       = vals.Numbers(-1.0, 1.0),
-                set_parser = float,
-                get_parser = float,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "trig_lvl_acq_path_1_float"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "trig_lvl_acq_path_1_float")
-            )
-
-            self.add_parameter(
-                "sequencer{}_avg_mode_en_acq_path0".format(seq_idx),
-                label      = "Sequencer {} averaging mode enable for acquisition path 0.".format(seq_idx),
-                docstring  = "Sets/gets sequencer {} averaging mode enable for acquisition path 0.".format(seq_idx),
+                "sequencer{}_demod_en_acq".format(seq_idx),
+                label      = "Sequencer {} demodulation enable for acquisition.".format(seq_idx),
+                docstring  = "Sets/gets sequencer {} demodulation enable for acquisition.".format(seq_idx),
                 unit       = '',
                 vals       = vals.Bool(),
                 set_parser = bool,
                 get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "avg_en_acq_path_0"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "avg_en_acq_path_0")
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "demod_en_acq"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "demod_en_acq")
             )
 
             self.add_parameter(
-                "sequencer{}_avg_mode_en_acq_path1".format(seq_idx),
-                label      = "Sequencer {} averaging mode enable for acquisition path 1.".format(seq_idx),
-                docstring  = "Sets/gets sequencer {} averaging mode enable for acquisition path 1.".format(seq_idx),
+                "sequencer{}_integration_length_acq".format(seq_idx),
+                label      = "Sequencer {} integration length for acquisition paths 0 and 1".format(seq_idx),
+                docstring  = "Sets/gets sequencer {} integration length in number of samples for non-weighed acquisitions on paths 0 and 1. Must be a multiple of 4".format(seq_idx),
                 unit       = '',
-                vals       = vals.Bool(),
-                set_parser = bool,
-                get_parser = bool,
-                set_cmd    = pulsar_qrm._gen_set_func_par(self._set_sequencer_config_val, seq_idx, "avg_en_acq_path_1"),
-                get_cmd    = pulsar_qrm._gen_get_func_par(self._get_sequencer_config_val, seq_idx, "avg_en_acq_path_1")
+                vals       = vals.Multiples(4, min_value=4, max_value=2**24-4),
+                set_parser = int,
+                get_parser = int,
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "non_weighed_integration_len"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "non_weighed_integration_len")
             )
 
+            self.add_parameter(
+                "sequencer{}_phase_rotation_acq".format(seq_idx),
+                label      = "Sequencer {} integration result phase rotation".format(seq_idx),
+                docstring  = "Sets/gets sequencer {} integration result phase rotation in degrees.".format(seq_idx),
+                unit       = 'Degrees',
+                vals       = vals.Numbers(0, 360),
+                set_parser = float,
+                get_parser = float,
+                set_cmd    = partial(self._set_sequencer_config_rotation_matrix, seq_idx), 
+                get_cmd    = partial(self._get_sequencer_config_rotation_matrix, seq_idx)
+            )
+
+            self.add_parameter(
+                "sequencer{}_discretization_threshold_acq".format(seq_idx),
+                label      = "Sequencer {} discretization threshold".format(seq_idx),
+                docstring  = "Sets/gets sequencer {} discretization threshold for discretizing the phase rotation result. ".format(seq_idx) + \
+                             "Discretization is done by comparing the threshold to the rotated integration result of path 0. " + \
+                             "This comparison is applied before normalization (i.e. division) of the rotated value " + \
+                             "with the integration length and therefore the threshold needs to be compensated " + \
+                             "(i.e. multiplied) with this length for the discretization to function properly.",
+                unit       = '',
+                vals       = vals.Numbers(-1.0*(2**24-4), 1.0*(2**24-4)),
+                set_parser = float,
+                get_parser = float,
+                set_cmd    = partial(self._set_sequencer_config_val, seq_idx, "discr_threshold"),
+                get_cmd    = partial(self._get_sequencer_config_val, seq_idx, "discr_threshold")
+            )
+
+    #--------------------------------------------------------------------------
+    def _set_acq_scope_config_val(self, param, val):
+        """
+        Set value of specific scope acquisition parameter.
+
+        Parameters
+        ----------
+        param : str
+            Parameter name.
+        val
+            Value to set parameter to.
+
+        Returns
+        ----------
+
+        Raises
+        ----------
+        Exception
+            Invalid input parameter type.
+        Exception
+            An error is reported in system error and debug <= 1.
+            All errors are read from system error and listed in the exception.
+        """
+
+        try:
+            self._set_acq_scope_config({param: val})
+        except:
+            raise
+
+    #--------------------------------------------------------------------------
+    def _get_acq_scope_config_val(self, param):
+        """
+        Get value of specific scope acquistion parameter.
+
+        Parameters
+        ----------
+        param : str
+            Parameter name.
+
+        Returns
+        ----------
+        val
+            Parameter value.
+
+        Raises
+        ----------
+        Exception
+            Invalid input parameter type.
+        Exception
+            An error is reported in system error and debug <= 1.
+            All errors are read from system error and listed in the exception.
+        """
+
+        try:
+            return self._get_acq_scope_config()[param]
+        except:
+            raise
+
     #--------------------------------------------------------------------------
     def _set_sequencer_config_val(self, sequencer, param, val):
         """
@@ -547,67 +683,22 @@ class pulsar_qrm_qcodes(pulsar_qrm_ifc, Instrument):
             with open(file_name, 'r') as file:
                 wave_and_prog_dict = json.load(file)
                 validate(wave_and_prog_dict, self._wave_and_prog_json_schema)
-                target_list = ["awg", "acq"]
-                for target in target_list:
-                    if "target" in wave_and_prog_dict["waveforms"]:
-                        for name in wave_and_prog_dict["waveforms"][target]:
-                            validate(wave_and_prog_dict["waveforms"][target][name], self._wave_json_schema)
+                for name in wave_and_prog_dict["waveforms"]:
+                    validate(wave_and_prog_dict["waveforms"][name], self._wave_json_schema)
+                for name in wave_and_prog_dict["weights"]:
+                    validate(wave_and_prog_dict["weights"][name], self._wave_json_schema)
+                for name in wave_and_prog_dict["acquisitions"]:
+                    validate(wave_and_prog_dict["acquisitions"][name], self._acq_json_schema)
                 self._delete_waveforms(sequencer)
                 self._add_waveforms(sequencer, wave_and_prog_dict["waveforms"])
+                self._delete_weights(sequencer)
+                self._add_weights(sequencer, wave_and_prog_dict["weights"])
+                self._delete_acquisitions(sequencer)
+                self._add_acquisitions(sequencer, wave_and_prog_dict["acquisitions"])
                 self._set_sequencer_program(sequencer, wave_and_prog_dict["program"])
         except:
             raise
 
-    #--------------------------------------------------------------------------
-    @staticmethod
-    def _gen_set_func_par(func, *args):
-        """
-        Generate set function with fixed parameters.
-
-        Parameters
-        ----------
-        func : callable
-            Function to generate set function for.
-        args
-            Function arguments.
-
-        Returns
-        ----------
-
-        Raises
-        ----------
-        """
-
-        def set_func(val):
-            return func(*args, val)
-        return set_func
-
-    #--------------------------------------------------------------------------
-    @staticmethod
-    def _gen_get_func_par(func, *args):
-        """
-        Generate get function with fixed parameters.
-
-        Parameters
-        ----------
-        func : callable
-            Function to generate set function for.
-        args
-            Function arguments.
-
-        Returns
-        ----------
-        val
-            Function return value.
-
-        Raises
-        ----------
-        """
-
-        def get_func():
-            return func(*args)
-        return get_func
-
 #-- class ---------------------------------------------------------------------
 
 class pulsar_qrm(pulsar_qrm_qcodes):
@@ -675,7 +766,7 @@ class pulsar_qrm_dummy(pulsar_qrm_qcodes):
         """
 
         #Create transport layer (socket interface)
-        transport_inst = pulsar_dummy_transport(pulsar_qrm_ifc._get_sequencer_cfg_format())
+        transport_inst = pulsar_dummy_transport(pulsar_qrm_ifc._get_acq_scope_config_format(), pulsar_qrm_ifc._get_sequencer_cfg_format())
 
         #Initialize parent classes.
         super(pulsar_qrm_dummy, self).__init__(name, transport_inst, debug)
