@@ -22,9 +22,9 @@ def get_build_info():
     ----------
     """
 
-    return {"version": "0.3.2",
-            "date":    "21/04/2021-16:38:33",
-            "hash":    "0x94E811E5",
+    return {"version": "0.4.0",
+            "date":    "21/07/2021-18:27:27",
+            "hash":    "0x70593743",
             "dirty":   False}
 
 
