@@ -299,6 +299,11 @@ class s4g_dac_channel(InstrumentChannel):
             get_cmd=partial(self._api.get_stepsize, dac),
             docstring="Returns the smallest current step allowed by the dac for the current settings.",
         )
+        self.add_parameter('dac_channel',
+                           set_cmd=False,
+                           get_cmd=lambda: dac,
+                           docstring="Returns the dac number of this channel."
+                           )
 
     def _get_span(self, dac: int):
         """
